#include <iom128v.h>
#include <ina90.h>

#define UART_BAUDRATE 23 // 19200 @ 7.3728 MHz
#include "uart.h"

#define AT45DB642 // chip type
#include "at45.h"

unsigned int failaddr;

void mem_init(void)
{
	AT45_init();
}

void mem_erase(unsigned int page)
{
	init_SPI_45();
	wait_ready_45();
	clear_CS_45();
	write_SPI_45(CMD_PAGE_ERASE);
	write_SPI_45((char)(page>>5));
	write_SPI_45((char)(page<<3));
	write_SPI_45(0x00);
	set_CS_45();
	wait_ready_45();
	close_SPI_45();
}

void mem_program(unsigned int page, unsigned char data)
{
	unsigned int i;
	AT45_write_buffer_begin(0);
	for(i = 0; i < AT45_PAGE_SIZE; i++)
		AT45_write(data);
	AT45_write_buffer_end();
	AT45_buf_to_mem(page);
}

unsigned char mem_verify(unsigned int page, unsigned char data)
{
	unsigned int i;
	unsigned char ok,x;
	ok = 1;
	AT45_read_mem_begin(page);
	for(i = 0; i < AT45_PAGE_SIZE; i++)
	{
		x = AT45_read();
		if(x != data)
		{
			if(ok)
			{
				failaddr = i;
				ok = 0;
			}
		}
	}
	AT45_read_mem_end();
	return ok;
}

unsigned char buf[66];    

void mem_read(unsigned int page)
{
	unsigned int i,l;
	unsigned char x;
	AT45_read_mem_begin(page);
	//for(i = 0; i < AT45_PAGE_SIZE; i++)
	for(i = 0; i < 16; i++)
	{
		for(l = 0; l < 66; l++)
			buf[l] = AT45_read();
		if(i)
			UART_read();
		for(l = 0; l < 66; l++)
			UART_write(buf[l]);
	}
	AT45_read_mem_end();
}

void main(void)
{
	unsigned char data,c,ok,temp;
	unsigned int i;
	
	UART_init(); // uart
	
	mem_init();
	
	for(;;)
	{
		data = UART_read();
		
		if(data == '$') // logo
		{
			UART_write('m');
			UART_write('e');
			UART_write('m');
			UART_write('3');
		}
		else if(data == 'T') // logo
		{                
            temp = AT45_get_status();
            UART_write(temp);
			UART_write(0x0d);
		}
		else if(data == '-') // erase
		{
			i = UART_read();
			i = (i<<8)|UART_read();
			mem_erase(i);
			UART_write(0x0d);
		}
		else if(data == '>') // program
		{
			i = UART_read();
			i = (i<<8)|UART_read();
			c = UART_read();
			mem_program(i,c);
			UART_write(0x0d);
		}
		else if(data == '|') // verify
		{
			i = UART_read();
			i = (i<<8)|UART_read();
			c = UART_read();
			ok = mem_verify(i,c);
			UART_write(failaddr>>8);
			UART_write(failaddr);
			UART_write(ok ? 0x0d : 0x1b);
		}
		else if(data == '<') // read
		{
			i = UART_read();
			i = (i<<8)|UART_read();
			mem_read(i);
			UART_write(0x0d);
		}
	}
}